------------------------------ MODULE FastMPGC ------------------------------

(* Fast Modes *) 

EXTENDS FiniteSets, Naturals, Library, Generic, TLCDefs

CONSTANTS Initial, Acceptor

\*  RespQuorum is the set of quorums used to determine a response
\*  AbortQuorum is the set of quorums used to determine an abort value
\*  AbortValues([Q -> S]) is the set of safe abort values given 
\*  the dStates of a quorum Q of acceptors.
CONSTANTS RespQuorum, AbortQuorum

INSTANCE FastMPGCDefs

VARIABLES status, pending, initVals, dState, accStatus, interface, 
  abortVals, pastPending

INSTANCE MPGC

slin_status == [p \in P |-> IF status[p] \in {"pending","panic"} THEN "pending" ELSE status[p]]
slin_pending == pending
slin_dState == Max({GLB(SrvStates(Q)) : Q \in RespQuorum}, LAMBDA a,b : a \preceq b)
slin_interface == interface
slin_initialized == 
  IF Initial THEN TRUE
  ELSE \E Q \in RespQuorum : \A a \in Q : accStatus[a] # "idle"
slin_initVals == initVals
slin_abortVals == abortVals

SLin == INSTANCE SpecLin WITH
    status <- slin_status,
    pending <- slin_pending,
    dState <- slin_dState,
    interface <- slin_interface,
    initialized <- slin_initialized,
    initVals <- slin_initVals,
    abortVals <- slin_abortVals

THEOREM Spec => SLin!Spec

=============================================================================
\* Modification History
\* Last modified Mon Dec 09 10:11:38 CET 2013 by nano
\* Last modified Sun Dec 08 19:42:57 CET 2013 by losa
\* Created Fri Nov 29 14:19:05 CET 2013 by losa
