---------------------------- MODULE FastMPGCDefs ----------------------------

EXTENDS FiniteSets, Library

CONSTANTS _ \preceq _, GLB(_)

CONSTANTS RespQuorum, AbortQuorum, Acceptor


ASSUME \A Q,R \in RespQuorum : Q \intersect R # {}
ASSUME \A Q \in RespQuorum, R \in AbortQuorum : 
  Cardinality(Q \intersect R) \geq (Cardinality(R) \div 2) + 1

\*  Examples of quorums:
N == Cardinality(Acceptor)
RespQuorum1 ==
  {Q \in SUBSET Acceptor : 
    Cardinality(Q) \geq ((2*N) \div 3) + 1}
AbortQuorum1 == RespQuorum1
RespQuorum2 ==
  {Q \in SUBSET Acceptor : 
    Cardinality(Q) \geq ((3*N) \div 4) + 1}
AbortQuorum2 ==
  {Q \in SUBSET Acceptor : 
    Cardinality(Q) \geq (N \div 2) + 1}
RespQuorum3 == {Acceptor}
AbortQuorum3 == {{a} : a \in Acceptor}


RemovePrefixes(ss) ==
  {s \in ss : \neg (\E s1 \in ss \ {s} : s \preceq s1)}

Majority(Q) ==
  {Maj \in SUBSET Q :
    Cardinality(Maj) \geq Cardinality(Q) \div 2 + 1}

\*  Assumes DStates is a function [Q -> S] where S is an abort quorum.
AbortValues(DStates) ==
  LET Q == DOMAIN DStates
      MajSets == {Image([a \in Maj |-> DStates[a]]) : Maj \in Majority(Q)}
      G == { GLB(ss) : ss \in MajSets}
  IN  RemovePrefixes(G)

=============================================================================
\* Modification History
\* Last modified Wed Dec 04 10:30:52 CET 2013 by losa
\* Created Fri Nov 29 16:35:06 CET 2013 by losa
