------------------------------ MODULE Library ------------------------------

EXTENDS Sequences, Naturals, FiniteSets

Last(s) == s[Len(s)]

Some(S) == CHOOSE e \in S : TRUE

Min(i,j) == IF i < j THEN i ELSE j
Max(S, LessEq(_,_)) == CHOOSE e \in S : \A e1 \in S : LessEq(e1,e)

Image(f) == {f[x] : x \in DOMAIN f}

\* TODO: When Seq is not overriden, TLC crashes with a strange error instead of indicating that Seq(E) is not enumerable (when model-checking the shared mem consensus).
NoDupSeq(E) == {es \in Seq(E) : \A i,j \in DOMAIN es : es[i] = es[j] => i = j}
\* A more efficient test:
RECURSIVE NoDup(_,_)
NoDup(es, seen) == 
    IF es = <<>> 
    THEN TRUE 
    ELSE
        IF es[1] \in seen 
        THEN FALSE 
        ELSE NoDup(Tail(es), seen \cup {es[1]})
NoDupSeq1(E) == {es \in Seq(E) : NoDup(es,{})}

RECURSIVE RemDupRec(_,_)
RemDupRec(es, seen) ==
  IF es = <<>>
  THEN <<>>
  ELSE
    IF es[1] \in seen
    THEN RemDupRec(Tail(es), seen)
    ELSE <<es[1]>> \o RemDupRec(Tail(es), seen \cup {es[1]}) 
RemDup(es) == RemDupRec(es, {})

\* xs is expected to be a sequence without duplicate elements and containing x
Truncate(x,xs) ==
    LET n == CHOOSE i \in DOMAIN xs : xs[i] = x
    IN SubSeq(xs,1,n)
\* A more efficient version:
RECURSIVE TruncateRec(_,_,_)
TruncateRec(x,xs,n) ==
    IF xs[n] = x 
    THEN [i \in 1..n |-> xs[i]]
    ELSE TruncateRec(x,xs,n+1)
Truncate1(x,xs) == TruncateRec(x,xs,1)

Map(f,op(_)) == [x \in DOMAIN f |-> op(f[x])]

DifferAt(i,s1,s2) == IF i <= Len(s1) /\ i <= Len(s2) THEN s1[i] /= s2[i] ELSE TRUE
CommonPrefixLength(s1,s2) == 
    LET min == Min(Len(s1),Len(s2)) IN
        IF min = 0 THEN 0 
        ELSE
            IF s1[1] /= s2[1] THEN 0
            ELSE CHOOSE i \in 1..min : (\A j \in 1..i : s1[j] = s2[j]) /\ DifferAt(i+1,s1,s2)

Difference(s1,s2,E) ==  
    LET i == CommonPrefixLength(s1,s2) IN 
        IF i >= Len(s2) THEN {} ELSE {e \in E : \E j \in (i+1)..Len(s2) : e = s2[j] }
\* A more efficient version:
RECURSIVE CommonLenRec(_,_,_)
CommonLenRec(s1, s2, i) ==
    IF i > Min(Len(s1),Len(s2))
    THEN i-1
    ELSE
        IF s1[i] /= s2[1]
        THEN i-1
        ELSE CommonLenRec(s1,s2,i+1)
Difference1(s1, s2) ==
    LET n == CommonLenRec(s1,s2,1)
    IN  [i \in 1..(Len(s2)-n) |-> s2[i+n]]

\* s1 is a prefix of s2
Prefix(s1,s2) == 
    /\  Len(s1) <= Len(s2)
    /\  \A i \in DOMAIN s1 : s1[i] = s2[i]

RECURSIVE MinRec(_,_)
MinRec(S, min) == 
    IF S = {}
    THEN min
    ELSE
        LET e == Some(S)
            newset == S \ {e}
        IN  IF e < min
        THEN MinRec(S, e)
        ELSE MinRec(S, min)
        
MinSet(S) == MinRec(S,0)

RECURSIVE LongestCommonPrefixLenRec(_,_,_)
LongestCommonPrefixLenRec(S,n,e1) ==
    IF S = {}
    THEN 0
    ELSE
        IF /\ \A e \in S : Len(e) >= n+1
           /\ \A e \in S : e[n+1] = e1[n+1]
        THEN LongestCommonPrefixLenRec(S,n+1,e1)
        ELSE n
    
LongestCommonPrefixLenSet(S) == LongestCommonPrefixLenRec(S, 0, Some(S))

LongestCommonPrefix(S) ==
    LET n == LongestCommonPrefixLenSet(S)
    IN  IF n = 0
        THEN <<>>
        ELSE [i \in 1..LongestCommonPrefixLenSet(S) |-> Some(S)[i]]
    
Diff(s1,s2) ==
    IF Len(s1) < Len(s2)
    THEN [i \in 1..(Len(s2)-Len(s1)) |-> s2[Len(s1)+i]]
    ELSE <<>>

RECURSIVE TakeWhile(_,_)
TakeWhile(test(_),s) ==
    IF s = <<>>
    THEN <<>>
    ELSE
        IF test(s[1])
        THEN <<s[1]>> \o TakeWhile(test,Tail(s))
        ELSE <<>>
        
=============================================================================
\* Modification History
\* Last modified Mon Dec 09 10:12:46 CET 2013 by nano
\* Last modified Thu Dec 05 10:49:14 CET 2013 by losa
\* Created Mon Aug 05 15:49:10 CEST 2013 by losa
