-------------------------- MODULE Linearizability --------------------------

EXTENDS RDR

VARIABLES
    status, pending, dState, nxtOut, interface

INSTANCE LinInterface

vars == <<status, pending, dState, nxtOut, interface>>

Label == {"ready","committed","pending"} \* The status of a process.

TypeInvariant == 
    \A p \in P :
        /\ status[p] \in Label
        /\ pending[p] \in C
        /\ nxtOut[p] = O
    /\ dState \in S
    
\* Invocation by process p:
Inv(p) == \E c \in C :
    /\ status[p] = "ready"
    /\ status' = [status EXCEPT ![p] = "pending"]
    /\ pending' = [pending EXCEPT ![p] = <<p,c>>]
    /\ Invoke(p, c)
    /\ UNCHANGED <<dState, nxtOut>>

\* Response by process p:
Resp(p) ==
    /\  status[p] = "committed"
    /\  status' = [status EXCEPT ![p] = "ready"]
    /\  Response(p, nxtOut[p])
    /\  UNCHANGED <<dState, pending, nxtOut>>

\* Linearize one pending request.
Lin ==
    /\ \E p \in P :
        /\  status[p] = "pending" 
        /\  status' = [status EXCEPT ![p] = "committed"]
        /\  dState' = dState \bullet pending[p]
        /\  nxtOut' = [nxtOut EXCEPT ![p] = Output(dState,pending[p])]
    /\ UNCHANGED <<pending, interface>>

Init ==
    /\ status = [p \in P |-> "ready"]
    /\ dState = Bot
    /\ pending = [p \in P |-> Some(Req)]
    /\ nxtOut = [p\in P |-> Some(O)]
    /\ interface = InterfaceInit
    
Next == Lin \/ (\E p \in P : Inv(p) \/ Resp(p))

Spec == Init /\ [][Next]_vars


=============================================================================
\* Modification History
\* Last modified Fri Nov 29 20:19:08 CET 2013 by losa
\* Last modified Wed Nov 13 16:11:38 CET 2013 by nano
\* Created Thu Aug 15 10:07:40 CEST 2013 by losa
