------------------------------ MODULE Network ------------------------------

CONSTANT Msg, Agent

Packet == [from:Agent, to:Agent, msg:Msg]
MkPacket(src,m,dst) == [
    from |-> src,
    msg |-> m,
    to |-> dst ]

VARIABLE network

\*  Async channels with message loss 
\*  but no duplication or corruption (e.g. TCP).

Packets(src, ms) == 
    UNION { { MkPacket(src,m,dst) : m \in ms[dst] } : dst \in DOMAIN ms }
\* ms must be a function from destination to set of messages.
Snd(src, ms) ==
    network' = network \cup Packets(src, ms)

Rcv(dst, m, src) ==
    LET packet == MkPacket(src,m,dst)
    IN  /\ packet \in network
        /\ network' = network \ {packet}

\* Receive and reply at once, to simplify.
RcvSnd(dst, m, src, ms) ==
    LET packet == MkPacket(src,m,dst)
    IN  /\ packet \in network
        /\ network' = (network \ {packet}) \cup Packets(dst, ms)

NetworkInvariant ==
  \A packet \in network :
    packet \in Packet

=============================================================================
\* Modification History
\* Last modified Fri Nov 29 17:16:46 CET 2013 by losa
\* Created Fri Nov 15 15:32:36 CET 2013 by losa
