------------------------------ MODULE SafeMPGC ------------------------------

(*  Safe Modes *)

EXTENDS FiniteSets, Naturals, Library, Generic, TLCDefs

CONSTANTS Initial, Acceptor

INSTANCE SafeMPGCDefs
    
VARIABLES status, pending, initVals, dState, accStatus, interface, 
  abortVals, pastPending

\*  In safe algorithms, the acceptors cannot become inconsistent. This can
\*  be implemented with a leader, or otherwise.
AcceptorConsistency ==
  \A acc1,acc2 \in Acceptor :
    LET s1 == dState[acc1]
        s2 == dState[acc2]
    IN s1 \preceq s2 \/ s2 \preceq s1

INSTANCE MPGC

ConsistentSpec == Init /\ [][Next /\ AcceptorConsistency']_vars

slin_status == [p \in P |-> IF status[p] \in {"pending","panic"} THEN "pending" ELSE status[p]]
slin_pending == pending
slin_dState == Max({GLB(SrvStates(Q)) : Q \in RespQuorum}, LAMBDA a,b : a \preceq b)
slin_interface == interface
slin_initialized == 
  IF Initial THEN TRUE
  ELSE \E Q \in RespQuorum : \A a \in Q : accStatus[a] # "idle"
slin_initVals == initVals
slin_abortVals == abortVals

SLin == INSTANCE SpecLin WITH
    status <- slin_status,
    pending <- slin_pending,
    dState <- slin_dState,
    interface <- slin_interface,
    initialized <- slin_initialized,
    initVals <- slin_initVals,
    abortVals <- slin_abortVals

THEOREM ConsistentSpec => SLin!Spec

=============================================================================
\* Modification History
\* Last modified Sun Dec 08 19:44:01 CET 2013 by losa
\* Created Fri Nov 29 16:01:28 CET 2013 by losa
