------------------------- MODULE SharedMemConsensus -------------------------

EXTENDS Library, Consensus, TLCDefs

INSTANCE RDR

\* local variables start with an underscore.
VARIABLES
    v, d, contention, pending, pc,
    interface,
    spinterface,
    abortVals \* ghost variable

VARIABLES splitterPc, x, y

INSTANCE SpecLinInterface
INSTANCE SplitterConcreteInterface

Splitter == INSTANCE Splitter WITH
  interface <- spinterface,
  pc <- splitterPc
   
splitterVars == <<splitterPc, x, y, spinterface>>


vars == <<v, d, contention, pending, pc, interface, spinterface, abortVals>>

TypeInvariant ==
    /\ pc \in [P -> {"L1","L2","L3","L4","L5","L6","L7","L8","L9",
            "L10","COMMITTED", "ABORTED"}]
    /\ pending \in [P -> Req]
    /\ v \in S
    /\ d \in BOOLEAN
    /\ contention \in BOOLEAN

Init ==
    /\ pc = [p \in P |-> "L1"]
    /\ d = FALSE
    /\ v = Bot
    /\ contention = FALSE
    /\ pending = [p \in P |-> Some(Req)]
    /\ interface = InterfaceInit
    /\ abortVals = {}
    
PCFromTo(p,l1,l2) ==
    /\ pc[p] = l1
    /\ pc' = [pc EXCEPT ![p] = l2]

Return(p,o) ==
    /\ pc' = [pc EXCEPT ![p] = "COMMITTED"]
    /\ Response(p,o)

GiveUp(p,av) == 
    /\ pc'  = [pc EXCEPT ![p] = "ABORTED"]
    /\ Abort(p, pending[p][2], av)
    /\ abortVals' = abortVals \cup {av}
    
Step1(p) ==
    /\  pc[p] = "L1"
    /\  pc' = [pc EXCEPT ![p] = "L2"]
    /\  \E c \in C : 
            /\ Invoke(p, c)
            /\ pending' = [pending EXCEPT ![p] = <<p,c>>]
    /\  UNCHANGED <<v,d,contention,spinterface, abortVals>>
 
Step2(p) == 
    /\  pc[p] = "L2"
    /\  IF  d = TRUE
        THEN
            IF \neg contention
            THEN 
              /\ Return(p,v)
              /\ UNCHANGED abortVals
            ELSE GiveUp(p,v)
        ELSE    
            /\ pc' =  [pc EXCEPT ![p] = "L3"]
            /\ UNCHANGED <<interface, abortVals>>
    /\  UNCHANGED <<v,d,contention, pending,spinterface>>

Step3a(p) ==
    /\ pc[p] = "L3"
    /\ InvokeSplitter(p,spinterface,spinterface')
    /\ UNCHANGED <<v,d,contention, pending, pc, interface, abortVals>>

Step3b(p) ==
    /\ \E b \in BOOLEAN : 
        /\  SplitterResponse(p,b,spinterface,spinterface')
        /\  IF b
            THEN pc' = [pc EXCEPT ![p] = "L4"]
            ELSE pc' = [pc EXCEPT ![p] = "L9"]
    /\ UNCHANGED <<v,d,contention, pending, interface, abortVals>>

Step4(p) ==
    /\ PCFromTo(p,"L4","L5")
    /\ v' = pending[p][2]
    /\ UNCHANGED <<d,contention, pending, interface, spinterface, abortVals>>
    
Step5(p) == 
    /\ pc[p] = "L5"
    /\  IF  \neg contention
        THEN    
            /\ pc' = [pc EXCEPT ![p] = "L6"]
        ELSE 
            /\ pc' =  [pc EXCEPT ![p] = "L8"]
    /\ UNCHANGED <<v,d,contention, pending, interface, spinterface, abortVals>>

Step6(p) ==
    /\ PCFromTo(p,"L6","L7")
    /\ d' = TRUE
    /\ UNCHANGED <<v,contention,pending,interface, spinterface, abortVals>>

Step7(p) ==
    /\ pc[p] = "L7"
    /\ Return(p,v)
    /\ UNCHANGED <<v,d,contention,pending,spinterface, abortVals>>

Step8(p) == 
    /\ pc[p] = "L8"
    /\ GiveUp(p, Bot)
    /\ UNCHANGED <<v,d,contention,pending,spinterface>>

Step9(p) ==
    /\ PCFromTo(p,"L9","L10")
    /\ contention' = TRUE 
    /\ UNCHANGED <<v,d,pending,interface,spinterface, abortVals>>

Step10(p) ==
    /\ pc[p] = "L10"
    /\ GiveUp(p,v)
    /\ UNCHANGED <<v,d,contention,pending,spinterface>>

Next == \E p \in P :
    \/ Step1(p) \/ Step2(p) \/ Step3a(p) \/ Step3b(p) \/ Step4(p) \/ Step5(p) \/ Step6(p) \/ Step7(p)
    \/ Step8(p) \/ Step9(p) \/ Step10(p)

NextComp ==
  /\  \/ Next
      \/ UNCHANGED vars
  /\  \/ Splitter!Next
      \/ x' = x /\ y' = y /\ splitterPc' = splitterPc /\ spinterface' = spinterface

Spec == Init /\ Splitter!Init /\ [][NextComp]_<<vars, splitterVars>>

status == 
  [p\in P |-> 
    IF pc[p] \in {"L1","COMMITTED"} 
    THEN "ready"
    ELSE  IF pc[p] = "ABORTED"
          THEN "aborted"
          ELSE "pending" ]

dState == 
  IF \E p \in P : pc[p] \in {"L6","L7","COMMITTED"}
  THEN v
  ELSE Bot

SLin == INSTANCE SpecLin WITH
  Initial <- TRUE,
  pending <- pending,
  initialized <- TRUE,
  initVals <- {}

THEOREM Spec => SLin!Spec

=============================================================================
\* Modification History
\* Last modified Mon Dec 09 09:40:14 CET 2013 by losa
\* Last modified Mon Jul 15 15:32:36 CEST 2013 by nano
\* Created Thu Jul 04 12:11:36 CEST 2013 by nano
