------------------------ MODULE SpecLinCorrectness ------------------------

EXTENDS RDR

---------------------------- MODULE SpecLinIsLin ----------------------------

VARIABLE interface

Mode1(status, pending, dState, initialized, initVals, abortVals) == 
    INSTANCE SpecLin WITH Initial <- TRUE

Lin(status, pending, dState, nxtOut) == INSTANCE Linearizability WITH
    interface <- [inv |-> interface.inv, resp |-> interface.resp]

Mode1Spec == \EE status, pending, s, initVals, abortVals, initialized : 
    Mode1(status, pending, s, initVals, abortVals, initialized)!Spec
    
LinSpec == \EE status, pending, s, nxtOut : Lin(status, pending, s, nxtOut)!Spec

THEOREM Mode1Spec => LinSpec

=============================================================================

------------------------ MODULE SpecLinIsIdemPotent -------------------------

(*  Here we compose two instances of speculative linearizability using the
    method described in section 2.4.6. *)

EXTENDS SpecLinInterface

SingleMode(status, pending, dState, initVals, abortVals, initialized) == 
    INSTANCE SpecLin WITH Initial <- TRUE

---------------------------- MODULE Composition -----------------------------
    
VARIABLES status1, pending1, dState1, initVals1, abortVals1, initialized1, interface1
vars1 == <<status1, pending1, dState1, initVals1, abortVals1, initialized1, 
  interface1>>
Mode1 == INSTANCE SpecLin WITH
    Initial <- TRUE,
    status <- status1, pending <- pending1, dState <- dState1, initVals <- initVals1,
    abortVals <- abortVals1, 
    initialized <- initialized1, interface <- interface1
     
VARIABLES status2, pending2, dState2, initVals2, abortVals2, initialized2, interface2
vars2 == <<status2, pending2, dState2, initVals2, abortVals2, initialized2, 
  interface2>>
Mode2 == INSTANCE SpecLin WITH
    Initial <- FALSE,
    status <- status2, pending <- pending2, dState <- dState2, initVals <- initVals2,
    abortVals <- abortVals2, 
    initialized <- initialized2, interface <- interface2

LinkInterfaces ==
    /\  interface1'.abort = interface2'.init
    /\ \A p\in P :
        /\  interface1'.inv[p] # interface1.inv[p]
            => interface.inv' = [interface.inv EXCEPT ![p] 
                = [@ EXCEPT !.cmd = interface1'.inv[p].cmd, 
                            !.flag = \neg @]]
        /\  interface1'.resp[p] # interface1.resp[p]
            => interface.resp' = [interface.resp EXCEPT ![p] 
                = [@ EXCEPT !.output = interface1'.resp[p].output, 
                            !.flag = \neg @]]
        /\  interface2'.inv[p] # interface2.inv[p]
            => interface.inv' = [interface.inv EXCEPT ![p] 
                = [@ EXCEPT !.cmd = interface2'.inv[p].cmd, 
                            !.flag = \neg @]]
        /\  interface2'.resp[p] # interface2.resp[p]
            => interface.resp' = [interface.resp EXCEPT ![p] 
                = [@ EXCEPT !.output = interface2'.resp[p].output, 
                            !.flag = \neg @]]
    /\  interface1'.interface.inv = interface1.interface.inv 
    /\  interface2'.interface.inv = interface2.interface.inv 
        => interface.inv' = interface.inv
    /\  interface1'.interface.resp = interface1.interface.resp 
    /\  interface2'.interface.resp = interface2.interface.resp 
        => interface.resp' = interface.resp
    /\  interface1'.abort = interface2'.init
    
CompoNext == 
    /\  \/  /\  Mode1!Next
            /\  UNCHANGED vars2 
        \/  /\  Mode2!Next
            /\  UNCHANGED vars1 
        \/ \E p \in P : Mode1!Abo(p) /\ Mode2!Ini(p)
    /\  LinkInterfaces

CompoInit == Mode1!Init /\ Mode2!Init /\ interface = InterfaceInit

CompoSpec == CompoInit /\ [][CompoNext]_<<vars1,vars2,interface>>

=============================================================================

Compo(status1, pending1, dState1, initVals1, abortVals1, initialized1, interface1,
    status2, pending2, dState2, initVals2, abortVals2, initialized2, interface2) 
        == INSTANCE Composition

CompoSpec == 
    \EE status1, pending1, dState1, initVals1, abortVals1, initialized1, interface1 :
        \EE status2, pending2, dState2, initVals2, abortVals2, initialized2, interface2 : 
            Compo(status1, pending1, dState1, initVals1, abortVals1, initialized1, 
              interface1, status2, pending2, dState2, initVals2, abortVals2, 
                initialized2, interface2)!CompoSpec

SingleModeSpec ==
    \EE status, pending, dState, initVals, abortVals, initialized : 
        SingleMode(status, pending, dState, initVals, abortVals, initialized)!Spec

THEOREM CompoSpec => SingleModeSpec

=============================================================================


=============================================================================
\* Modification History
\* Last modified Sat Dec 07 09:55:02 CET 2013 by losa
\* Created Fri Nov 29 11:34:08 CET 2013 by losa
