------------------------ MODULE SpecLinIsIdem ------------------------

(*  A spec for model checking the idempotence property of SpecLin 
    It would have been much cleaner to use instantiation instead of copying
    the specification of SpecLin. However a strange bug in TLC makes 
    instantiation a hazard. *)

EXTENDS Generic, TLCDefs

INSTANCE RDR

(* The first instance *)

VARIABLES
    status1, pending1, dState1, initialized1, abortValdState1, initValdState1, interface1
    
I1 == INSTANCE SpecLin WITH 
  Initial <- TRUE,
  interface <- interface1,
  status <- status1,
  dState <- dState1,
  pending <- pending1,
  initialized <- initialized1,
  abortVals <- abortValdState1,
  initVals <- initValdState1 

vardState1 == <<status1, pending1, dState1, initialized1, abortValdState1, initValdState1, interface1>>

(* The second instance *)

VARIABLES
    status2, pending2, dState2, initialized2, abortValdState2, initValdState2, interface2
    
I2 == INSTANCE SpecLin WITH 
  Initial <- FALSE,
  interface <- interface2,
  status <- status2,
  dState <- dState2,
  pending <- pending2,
  initialized <- initialized2,
  abortVals <- abortValdState2,
  initVals <- initValdState2 

vardState2 == <<status2, pending2, dState2, initialized2, abortValdState2, initValdState2, interface2>>

(* Interface refinement, tricky because of the flags *)

VARIABLES inv, resp
LinkInterfaces ==
    /\ interface1'.abort = interface2'.init
    /\ \A p\in P :
        /\  interface1'.inv[p] # interface1.inv[p]
            => inv' = [inv EXCEPT ![p] 
                = [@ EXCEPT !.cmd = interface1'.inv[p].cmd, 
                            !.flag = \neg @]]
        /\  interface1'.resp[p] # interface1.resp[p]
            => resp' = [resp EXCEPT ![p] 
                = [@ EXCEPT !.output = interface1'.resp[p].output, 
                            !.flag = \neg @]]
        /\  interface2'.inv[p] # interface2.inv[p]
            => inv' = [inv EXCEPT ![p] 
                = [@ EXCEPT !.cmd = interface2'.inv[p].cmd, 
                            !.flag = \neg @]]
        /\  interface2'.resp[p] # interface2.resp[p]
            => resp' = [resp EXCEPT ![p] 
                = [@ EXCEPT !.output = interface2'.resp[p].output, 
                            !.flag = \neg @]]
    /\  interface1'.inv = interface1.inv /\  interface2'.inv = interface2.inv 
        => inv' = inv
    /\  interface1'.resp = interface1.resp /\  interface2'.resp = interface2.resp 
        => resp' = resp

(* The composition *)

Next == 
    /\  \/  /\  \E p \in P :  I1!Lin  \/ I1!Inv(p) \/ I1!Resp(p) \/ I1!Ini(p) \/ I1!Recover
            /\  UNCHANGED vardState2 
        \/  /\  \E p \in P :  I2!Lin  \/ I2!Inv(p) \/ I2!Resp(p) \/ I2!Ini(p) \/ I2!Recover
            /\  status1' = status1 /\ pending1' = pending1 /\ dState1' = dState1 /\  interface1' = interface1
            /\  abortValdState1' = abortValdState1 /\ initValdState1' = initValdState1
            /\  initialized1' = initialized1
        \/ \E p \in P : I1!Abo(p) /\ I2!Ini(p)
    /\ LinkInterfaces

interface == [ 
    init |-> I1!SwitchInterfaceInit,
    inv |-> inv,
    resp |-> resp,
    abort |-> interface2.abort]
    

Spec == 
    I1!Init /\ I2!Init /\ inv = I1!LI!InvInterfaceInit /\ resp = I1!LI!RespInterfaceInit
    /\ [][Next]_<<vardState1,vardState2,inv,resp>>

(* The refinement mapping *)

status == [p\in P |-> IF status1[p] # "aborted" THEN status1[p] ELSE status2[p]]
pending == [p\in P |-> IF status1[p] # "aborted" THEN pending1[p] ELSE pending2[p]]
dState == IF dState2 = Bot THEN dState1 ELSE dState2
initialized == TRUE
initVals == initValdState1
Initial == TRUE
abortVals == abortValdState2

SLin == INSTANCE SpecLin

THEOREM Spec => SLin!Spec


=============================================================================
\* Modification History
\* Last modified Fri Nov 29 20:35:36 CET 2013 by losa
\* Created Fri Nov 22 17:18:20 CET 2013 by losa
