------------------------------ MODULE Splitter ------------------------------

EXTENDS SplitterInterface, Library

CONSTANT P

VARIABLES
    x, y, pc,
    interface

vars == <<x,y,pc,interface>>
Labels == {"START","L1","L2","L3","L4","END"}

TypeInvariant ==
    /\ x \in P
    /\ y \in BOOLEAN
    /\ pc \in [P -> Labels]
    /\ interface \in SpInterfaceType

PCFromTo(p,l1,l2) ==
    /\ pc[p] = l1
    /\ pc' = [pc EXCEPT ![p] = l2]

Start(p) ==
    /\ pc[p] = "START"
    /\ InvokeSplitter(p,interface,interface')
    /\ pc' = [pc EXCEPT ![p] = "L1"]
    /\ UNCHANGED <<x,y>>

WriteX(p) ==
    /\ PCFromTo(p,"L1","L2")
    /\ x' = p
    /\ interface' = interface
    /\ UNCHANGED y
    \*/\ UNCHANGED <<y,interface>>

TestY(p) ==
    /\ pc[p] = "L2"
    /\ IF y = TRUE 
       THEN /\ SplitterResponse(p,FALSE,interface,interface') 
            /\ pc' = [pc EXCEPT ![p] = "END"]
       ELSE /\ pc' = [pc EXCEPT ![p] = "L3"]
            /\ interface' = interface 
    /\ UNCHANGED <<x,y>>

WriteY(p) ==
    /\ PCFromTo(p,"L3","L4")
    /\ y' = TRUE
    /\ interface' = interface
    /\ UNCHANGED x

TestX(p) ==
    /\ PCFromTo(p,"L4","END")
    /\ IF x = p 
       THEN SplitterResponse(p,TRUE,interface,interface')
       ELSE SplitterResponse(p,FALSE,interface,interface')
    /\ UNCHANGED <<x,y>>

Init ==
    /\ pc = [p\in P |-> "START"]
    /\ x = Some(P)
    /\ y = FALSE
    /\ interface = SpInterfaceInit

Next == \E p\in P :
    Start(p) \/ WriteX(p) \/ TestY(p) \/ WriteY(p) \/ TestX(p)

Spec == Init /\ [][Next]_vars

=============================================================================
\* Modification History
\* Last modified Sun Dec 01 00:22:46 CET 2013 by losa
\* Last modified Thu Jul 04 14:48:12 CEST 2013 by nano
\* Created Thu Jul 04 12:13:51 CEST 2013 by nano
